package com.rightanglegames.backtothepast;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.RoundedRectangle;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

/**
 * The HUD class responsible for showing score and so on.
 * Also the Game Maker like controller class which deals with input that should be handled for the world.
 * This way we avoid adding input dealing code to the WorldGameState class.
 * Just add some visible or invisible controller class to your world and handle input there.
 * 
 * @author Thomas
 *
 */
public class Hud extends Entity {

	private static final String CMD_NEXT = "nextLevel";
	private static final String CMD_PREVIOUS = "previousLevel";

	private AngelCodeFont font;
	
	public Hud(float x, float y) {
		super(x, y);
		this.name = "HUD";
		this.depth = BackToThePast.HUD_LAYER;
		this.font = ResourceManager.getAngelCodeFont("HildaSonnenschein32");

		this.define(CMD_NEXT, Input.KEY_N);
		this.define(CMD_PREVIOUS, Input.KEY_P);
	}
	
	public void render(GameContainer container, Graphics g) throws SlickException {
		Mage mage = Globals.player;
		RoundedRectangle backRect = new RoundedRectangle(x, y, container.getWidth() - 1, 40, 20);
		Color c = Color.blue;
		c.a = 0.3f;
		Font oldFont = g.getFont();
		g.setFont(font);
		g.setColor(c);
		g.fill(backRect);
		g.draw(backRect);
		g.setColor(Color.white);
		g.drawString("Health: " + mage.getHealth(), x+15, y+5);
		g.drawString("Mana: " + mage.getMana(), x+135, y+5);
		g.drawString("Score: " +((IngameState)world).getScore(), x+265, y+5);
		g.drawString("Collected: " + mage.getCollected(), x+395, y+5);
		
		g.setFont(oldFont);

	}

	public void update(GameContainer container, int delta) throws SlickException {
		if (check(CMD_NEXT)) {
			Globals.levelDone = true;
			Globals.playerCheated = true;
		}
		if (check(CMD_PREVIOUS)) {
			IngameState state = (IngameState) this.world;
			state.setLevelNo(state.getLevelNo() - 2);
			if (state.getLevelNo() < 0)
				state.setLevelNo(0);
			Globals.levelDone = true;
			Globals.playerCheated = true;
		}
	}
}
