package com.rightanglegames.backtothepast;

import it.randomtower.engine.World;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.util.ArrayList;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.util.ResourceLoader;
import org.newdawn.slick.util.pathfinding.PathFindingContext;
import org.newdawn.slick.util.pathfinding.TileBasedMap;

import com.rightanglegames.backtothepast.enemies.DummyRobot;
import com.rightanglegames.backtothepast.enemies.FlyingRobot;
import com.rightanglegames.backtothepast.enemies.WalkingRobot;


public class Level implements TileBasedMap {
	private static final char BLOCK = '=';
	private static final char MONK = 'P';
	private static final char CRATE = '#';
	private static final char DUMMYROBOT = 'r';
	private static final char WALKINGROBOT = 'w';
	private static final char FLYINGROBOT = 'f';
	private static final char TURRETUP = '^';
	private static final char TURRETDOWN = 'v';
	private static final char ACID = '~';
	private static final char EXIT = 'X';
	private static final char M = 'M';
	private static final char A = 'A';
	private static final char G = 'G';
	private static final char I = 'I';
	private static final char C = 'C';
	private static final char K = 'K';
	private static final char WAYPOINT1 = '1';
	private static final char WAYPOINT2 = '2';
	private static final char WAYPOINT3 = '3';
	private static final char WAYPOINT4 = '4';
	private static final char WAYPOINT5 = '5';
	private static final char WAYPOINT6 = '6';
	private static final char WAYPOINT7 = '7';
	private static final char WAYPOINT8 = '8';
	private static final char WAYPOINT9 = '9';
	private static final int NO_SOLID = -1;
	
	
	private ArrayList<String> lines = null;
	private int levelNo = 0;
	private int timeToFinish = 0;
	private ArrayList<String> description = new ArrayList<String>();
	private ArrayList<String> script = new ArrayList<String>();
	private Waypoint waypoints[] = new Waypoint[9];
	private int nrOfWaypoints = 0;
	private int blocked[][];
	private int widthInTiles = 0;
	private int heightInTiles = 0;

	public static Level load(int levelNo) throws SlickException {

		String level = Level.getLevelPath(levelNo);

		boolean levelExists = true;
		try {
			ResourceLoader.getResourceAsStream(level);
		} catch (RuntimeException e) {
			levelExists = false;
		}
		if (!levelExists) {
			return null;
		}

		BufferedReader reader = new BufferedReader(new InputStreamReader(ResourceLoader.getResourceAsStream(level)));
		Level loadedLevel = new Level();
		int width = 0;
		loadedLevel.lines = new ArrayList<String>();
		loadedLevel.setLevelNo(levelNo);
		
		// read through the lines recording them into a list and
		// determining the maximum width.
		int lineno = 0;
		try {
			boolean loadingLevelLines = true;
			boolean loadingMessage = false;
			boolean loadingScript = false;
			while (reader.ready()) {
				String line = reader.readLine();
				lineno ++;
				if (line == null) {
					break;
				}
				if ("ENDLEVEL".equalsIgnoreCase(line)) {
					loadingLevelLines = false;
					loadingMessage = true;
				} else if ("ENDDESCRIPTION".equalsIgnoreCase(line)) {
					loadingMessage = false;
					loadingScript = true;
				} else {
					if (loadingLevelLines) {
						width = Math.max(line.length(), width);
						loadedLevel.lines.add(line);
					} else if (loadingMessage){
						loadedLevel.description.add(line);
					} else if (loadingScript) {
						if (!"ENDSCRIPT".equalsIgnoreCase(line))
							loadedLevel.script.add(line);
					}
				}
			}
		} catch (IOException e) {
			throw new SlickException("Failed to load map: "+level, e);
		}
		
		int height = loadedLevel.lines.size();
		// create the array for the blocked information for this level
		loadedLevel.blocked = new int[width][height];
		// update the width and height of the world according to the level size
		loadedLevel.widthInTiles = width;
		loadedLevel.heightInTiles = height;
		return loadedLevel;
	}
	
	public static String getLevelPath(int levelno) {
		// now load the level file and create the entities from the file
		DecimalFormat formatter = new DecimalFormat("###000");
		String level = "levels/level" + formatter.format(levelno) + ".dat";
		return level;
	}

	public void createEntities(Level level, int width, int height, World world) throws SlickException {
//		Log.debug("createEntities: width = " + width + ", height = "+ height);
		int spriteIndex;
		
		nrOfWaypoints = 0;
		prepareBlockedArray(level, width, height, world);
		for (int x=0;x<width;x++) {
			for (int y=0;y<height;y++) {
//				Log.debug("createEntities: x = " + x + ", y = "+ y);
				char c = lines.get(y).charAt(x);
				float xpos = x * BackToThePast.TILESIZE;
				float ypos = y * BackToThePast.TILESIZE;
				switch (c) {
				case BLOCK:
					spriteIndex = blocked[x][y];
					world.add(new Block(xpos, ypos, BackToThePast.TILESIZE, BackToThePast.TILESIZE, spriteIndex));
					break;
				case MONK:
					world.add(new Mage(xpos, ypos));
					break;
				case CRATE:
					world.add(new Crate(xpos, ypos));
					break;
				case DUMMYROBOT:
					world.add(new DummyRobot(xpos, ypos));
					break;
				case WALKINGROBOT:
					world.add(new WalkingRobot(xpos, ypos));
					break;
				case FLYINGROBOT:
					world.add(new FlyingRobot(xpos, ypos));
					break;
				case ACID:
					world.add(new Acid(xpos, ypos));
					break;
				case EXIT:
					world.add(new LevelExit(xpos, ypos));
					break;
				case TURRETUP:
					world.add(new Turret(xpos, ypos, false));
					break;
				case TURRETDOWN:
					world.add(new Turret(xpos, ypos, true));
					break;
				case M:
					world.add(new Collectable("M", xpos, ypos));
					break;
				case A:
					world.add(new Collectable("A", xpos, ypos));
					break;
				case G:
					world.add(new Collectable("G", xpos, ypos));
					break;
				case I:
					world.add(new Collectable("I", xpos, ypos));
					break;
				case C:
					world.add(new Collectable("C", xpos, ypos));
					break;
				case K:
					world.add(new Collectable("K", xpos, ypos));
					break;
				case WAYPOINT1:
					waypoints[0] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT2:
					waypoints[1] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT3:
					waypoints[2] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT4:
					waypoints[3] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT5:
					waypoints[4] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT6:
					waypoints[5] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT7:
					waypoints[6] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT8:
					waypoints[7] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				case WAYPOINT9:
					waypoints[8] = new Waypoint(new Vector2f(x,y));
					nrOfWaypoints++;
					break;
				}
			}
		}
	}

	private void prepareBlockedArray(Level level, int width, int height,
			World world) {
		for (int x=0;x<width;x++) {
			for (int y=0;y<height;y++) {
				char c = lines.get(y).charAt(x);
				blocked[x][y] = NO_SOLID;
				switch (c) {
				case BLOCK:
					blocked[x][y] = 0;
					break;
				case TURRETUP:
				case TURRETDOWN:
					blocked[x][y] = 0;
					break;

				}
			}
		}
		// now second loop to check which neighbors are set
		for (int x=0;x<width;x++) {
			for (int y=0;y<height;y++) {
				if (blocked[x][y] != NO_SOLID) {
					if (y-1 >= 0 && blocked[x][y-1] == NO_SOLID)
						blocked[x][y] += 1;
					if (x+1 < width && blocked[x+1][y] == NO_SOLID)
						blocked[x][y] += 2;
					if (y+1 < height && blocked[x][y+1] == NO_SOLID)
						blocked[x][y] += 4;
					if (x-1 >= 0 && blocked[x-1][y] == NO_SOLID)
						blocked[x][y] += 8;
				}
			}
		}
		
	}

	public void setTimeToFinish(int timeToFinish) {
		this.timeToFinish = timeToFinish;
	}

	public int getTimeToFinish() {
		return timeToFinish;
	}

	public void setLevelNo(int levelNo) {
		this.levelNo = levelNo;
	}

	public int getLevelNo() {
		return levelNo;
	}

	@Override
	public int getWidthInTiles() {
		return widthInTiles;
	}

	@Override
	public int getHeightInTiles() {
		return heightInTiles;
	}

	@Override
	public void pathFinderVisited(int x, int y) {
	}

	public boolean blocked(Vector2f pos) {
		return blocked(pos.x, pos.y);
	}
	
	public boolean blocked(float x, float y) {
		int tx = (int) x / BackToThePast.TILESIZE;
		int ty = (int) y / BackToThePast.TILESIZE;
		return blocked(null, tx, ty);
	}
	@Override
	public boolean blocked(PathFindingContext context, int tx, int ty) {
		if (tx < 0 || tx >= widthInTiles)
			return false;
		if (ty < 0 || ty >= heightInTiles)
			return false;
		return blocked[tx][ty] != NO_SOLID;
	}

	@Override
	public float getCost(PathFindingContext context, int tx, int ty) {
		return 1;
	}
	
	
	public boolean saveToFile() {
		String level = Level.getLevelPath(this.getLevelNo());

        BufferedWriter bufferedWriter = null;
        try {
            
            //Construct the BufferedWriter object
            bufferedWriter = new BufferedWriter(new FileWriter(level));
            
            //Start writing to the output stream
            // first the level
            writeArrayList(lines, "ENDLEVEL", bufferedWriter);
            writeArrayList(description, "ENDDESCRIPTION", bufferedWriter);
            writeArrayList(script, "ENDSCRIPT", bufferedWriter);
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            return false;
        } catch (IOException ex) {
            ex.printStackTrace();
            return false;
        } finally {
            //Close the BufferedWriter
            try {
                if (bufferedWriter != null) {
                    bufferedWriter.flush();
                    bufferedWriter.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        return true;
	}
	
	private void writeArrayList(ArrayList<String> list, String endtoken, BufferedWriter bufferedWriter) throws IOException {
		if (list != null) {
	        for (String line : list) {
	            bufferedWriter.write(line);
	            bufferedWriter.newLine();
			}
		}
        bufferedWriter.write(endtoken);
        bufferedWriter.newLine();     
		
	}

	public ArrayList<String> getLines() {
		return lines;
	}

	public void setLines(ArrayList<String> lines) {
		this.lines = lines;
	}

	public ArrayList<String> getDescription() {
		return description;
	}

	public void setDescription(ArrayList<String> description) {
		this.description = description;
	}

	public ArrayList<String> getScript() {
		return script;
	}

	public void setScript(ArrayList<String> script) {
		this.script = script;
	}

	public void setWidthInTiles(int widthInTiles) {
		this.widthInTiles = widthInTiles;
	}

	public void setHeightInTiles(int heightInTiles) {
		this.heightInTiles = heightInTiles;
	}

	public Waypoint[] getWaypoints() {
		return waypoints;
	}

	public int getNrOfWaypoints() {
		return nrOfWaypoints;
	}
	
}
