package com.rightanglegames.backtothepast;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Alarm;
import it.randomtower.engine.entity.Entity;

public class LevelExit extends Entity {

	public static final String LEVEL_EXIT = "levelExit";
	
	private int frame = 0, row = 0, col = 0;
	private boolean exitActive = false;
	private Sound leaveLevel;
	
	public LevelExit(float x, float y) {
		super(x, y);
		sheet = ResourceManager.getSpriteSheet(LEVEL_EXIT);
		leaveLevel = ResourceManager.getSound("exit");
		currentImage = sheet.getSprite(row, col);
		depth = BackToThePast.OBJECT_LAYER;
		this.addType(LEVEL_EXIT);
		this.name = LEVEL_EXIT;
		this.setHitBox(0, 0, 48, 48);
		this.setAlarm("frame", BackToThePast.TARGET_FRAME_RATE / 10, false, true);
		this.setAlarm(LEVEL_EXIT, Alarm.FOREVER, true, true);
	}
	
	@Override
	public void update(GameContainer container, int delta) throws SlickException {
		super.update(container, delta);
		if (!exitActive)
			return;
		// collision with player - deal damage
		Entity mage = collide(Entity.PLAYER, x, y);
		if (mage != null) {
			// the exit is active and the player found us
			leaveLevel.play();
			Globals.levelDone = true;
		}
	}

	public void render(GameContainer container, Graphics g) throws SlickException {
		if (!exitActive)
			colorCurrentImage(Color.gray);
		else
			colorCurrentImage(Color.white);
		super.render(container, g);
	}
	
	private void colorCurrentImage(Color c) {
		currentImage.setColor(Image.TOP_LEFT, c.r, c.g, c.b, c.a);
		currentImage.setColor(Image.TOP_RIGHT, c.r, c.g, c.b, c.a);
		currentImage.setColor(Image.BOTTOM_LEFT, c.r, c.g, c.b, c.a);
		currentImage.setColor(Image.BOTTOM_RIGHT, c.r, c.g, c.b, c.a);
	}

	public void alarmTriggered(String name) {
		if (name.equals("frame")) {
			frame++;
			if (frame > 31) {
				frame = 0;
			}
			row = frame / 10;
			col = frame % 10;
			currentImage = sheet.getSprite(col, row);
		} else if (name.equals(LEVEL_EXIT)) {
			this.exitActive = true;
		}
	}
}
