package com.rightanglegames.backtothepast;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

public class SplashScreen extends EnhancedWorld {

	private static final String CMD_SKIP = "skip";
	private static final String CMD_WAIT = "wait";
	private BouncingLogo myLogo;
	private boolean presentsAdded = false;
	private int sleepTime = 0;
	private boolean firstTimeKeyPress = false;	// used for video capturing
	
	public SplashScreen(int id) {
		super(id);
		define(CMD_SKIP, Input.KEY_ESCAPE, Input.KEY_SPACE, Input.KEY_RETURN);
		define(CMD_WAIT, Input.KEY_W);
	}
	
	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		BackToThePast.initRessources();
		super.init(container, game);
	}

	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		this.clear();
		myLogo = new BouncingLogo(100.0f, -500.0f);
		this.add(myLogo);
	}

	public void render(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		// render all entities
		g.setBackground(Color.black);
		super.render(container, game, g);
	}

	public void update(GameContainer container, StateBasedGame game, int delta) throws SlickException {
		// uncomment the next 5 lines if we don't record a video
//		if (!firstTimeKeyPress) {
//			if (!check(CMD_WAIT))
//				return;
//			firstTimeKeyPress = true;
//		}
		// update all entities
		super.update(container, game, delta);
		if (check(CMD_SKIP))
			this.enterTitleState();
		if (myLogo.animationFinished() && !presentsAdded) {
			this.add(new AppearingPresents(BackToThePast.WIDTH / 2, BackToThePast.HEIGHT - 40));
			presentsAdded = true;
		}
		if (presentsAdded)
			sleepTime += delta;
		if (sleepTime > 3000) // three seconds
			enterTitleState();
	}

	public void enterTitleState() {
		Globals.game.enterState(BackToThePast.TITLE_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
	}


}
