package com.rightanglegames.backtothepast;

import it.randomtower.engine.ResourceManager;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

public class TitleState extends EnhancedWorld {

	private static final String CMD_PLAY = "play";
	private static final String CMD_EDITOR = "editor";
	private static final String CMD_QUIT = "quit";

	private static int DELAY_FOR_PAST = 2000;
	
	private static String message = "Press P to play, ESC to quit"; 
	
	private Image back, toTheFuture, past;
	private AngelCodeFont hilda;
	private int delayForPast = DELAY_FOR_PAST;
	
	public TitleState(int id) {
		super(id);
		define(CMD_QUIT, Input.KEY_ESCAPE);
		define(CMD_PLAY, Input.KEY_P, Input.KEY_SPACE, Input.KEY_RETURN);
		define(CMD_EDITOR, Input.KEY_E);
	}

	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		BackToThePast.initRessources();
		super.init(container, game);
		Globals.music = ResourceManager.getMusic("music");
		Globals.music.loop();	// start the background music
		back = ResourceManager.getImage("Back");
		back.setColor(Image.TOP_LEFT, Color.orange.r, Color.orange.g, Color.orange.b, Color.orange.a);
		back.setColor(Image.TOP_RIGHT, Color.orange.r, Color.orange.g, Color.orange.b, Color.orange.a);
		back.setColor(Image.BOTTOM_LEFT, Color.yellow.r, Color.yellow.g, Color.yellow.b, Color.yellow.a);
		back.setColor(Image.BOTTOM_RIGHT, Color.yellow.r, Color.yellow.g, Color.yellow.b, Color.yellow.a);
		toTheFuture = ResourceManager.getImage("ToTheFuture");
		toTheFuture.setColor(Image.BOTTOM_LEFT, Color.orange.r, Color.orange.g, Color.orange.b, Color.orange.a);
		toTheFuture.setColor(Image.BOTTOM_RIGHT, Color.orange.r, Color.orange.g, Color.orange.b, Color.orange.a);
		toTheFuture.setColor(Image.TOP_LEFT, Color.yellow.r, Color.yellow.g, Color.yellow.b, Color.yellow.a);
		toTheFuture.setColor(Image.TOP_RIGHT, Color.yellow.r, Color.yellow.g, Color.yellow.b, Color.yellow.a);
		past = ResourceManager.getImage("Past");
		hilda = ResourceManager.getAngelCodeFont("HildaSonnenschein32");
		this.delayForPast = DELAY_FOR_PAST;
	}

	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		this.clear();
	}
	public void render(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		// render all entities
		g.setBackground(Color.black);
		super.render(container, game, g);
		float x = BackToThePast.WIDTH / 2 - back.getWidth()/2;
		back.draw(x, 100);
		toTheFuture.draw(x, 170);
		if (delayForPast <= 0) {
			past.draw(x + 35, 115);
			g.setFont(hilda);
			g.drawString(message, BackToThePast.WIDTH / 2 - hilda.getWidth(message) / 2, 400);
		}
	}

	public void update(GameContainer container, StateBasedGame game, int delta) throws SlickException {
		if (delayForPast > 0)
			delayForPast -= delta;
		// update all entities
		super.update(container, game, delta);
		if (check(CMD_PLAY))
			Globals.game.enterState(BackToThePast.INGAME_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
		if (check(CMD_EDITOR))
			Globals.game.enterState(BackToThePast.EDITOR_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
		if (check(CMD_QUIT)) {
			container.exit();
		}
	}

}
