package com.rightanglegames.backtothepast.weapons;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;

import com.rightanglegames.backtothepast.Crate;
import com.rightanglegames.backtothepast.Mage;
import com.rightanglegames.backtothepast.effects.DebrisExplosion;
import com.rightanglegames.backtothepast.effects.Explosion;
import com.rightanglegames.backtothepast.enemies.Enemy;

import it.randomtower.engine.ME;
import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

public class Bullet extends Weapon {

	public enum BulletType { YELLOW, VIOLET };

	public static String BULLET = "bullet";
	public static String ANIMATEDBULLET = "animatedbullet";
	
	private BulletType type;
	private int frameRow = 0, frameCol = 0;
	private int nextFrame = ME.targetFrameRate / 3;
	private Sound explosionSnd, wallExplosionSnd;
	
	public Bullet(BulletType type, float x, float y, int damage, Vector2f speed, float life) {
		super(x, y, damage, speed, life);
		this.type = type;
		this.setHitBox(0, 0, 16, 16);
		explosionSnd = ResourceManager.getSound("explosion");
		wallExplosionSnd = ResourceManager.getSound("wallexplosion");
		switch (type) {
		case YELLOW:
			sheet = ResourceManager.getSpriteSheet(BULLET);
			break;
		case VIOLET:
			sheet = ResourceManager.getSpriteSheet(ANIMATEDBULLET);
			break;
		}
		this.addType(BULLET);
		this.currentImage = this.sheet.getSprite(frameCol, frameRow);
		this.setAlarm("frame", nextFrame, false, true);
	}

	@Override
	public void update(GameContainer container, int delta) throws SlickException {
		float explx;
		
		super.update(container, delta);
		// collision with player - deal damage
		Entity mage = collide(Entity.PLAYER, x, y);
		if (mage != null) {
			// player takes damage and we die in an explosion
			if (this.speed.x < 0)	// flying left
				explx = this.x - 40;
			else
				explx = this.x + this.width -16;
			this.world.add(new Explosion(explx, this.y-24,60));
			this.destroy();
			return;
		}
		// collision with crates - destroy this and crate in an explosion
		Entity crate = collide(Crate.CRATE, x, y);
		if (crate != null) {
			explosionSnd.play();
			if (this.speed.x < 0)	// flying left
				explx = this.x - 40;
			else
				explx = this.x + this.width -16;
			this.world.add(new DebrisExplosion(explx, this.y-24,60));
			this.destroy();
			return;
		}
		// collision with solid - destroy and create explosion
		Entity solid = collide(Entity.SOLID, x, y);
		if (solid != null && !solid.isType(Enemy.ENEMY)) {
			wallExplosionSnd.play();
			if (this.speed.x < 0)	// flying left
				explx = this.x - 40;
			else
				explx = this.x + this.width -16;
			this.world.add(new Explosion(explx, this.y-24, 60));
			this.destroy();
		}
	}

	@Override
	public void alarmTriggered(String name) {
		if (name.equals("frame")) {
			frameCol++;
			if (frameCol > 3) {
				frameCol = 0;
				frameRow++;
				if (frameRow > 3) {
					frameRow = 0;
				}
			}
			this.currentImage = this.sheet.getSprite(frameCol, frameRow);
		}
	}

}
